(function(angular, undefined) {
	"use strict";
	var $httpBackendRef,
		_defaultEndPointUrl = "/carouseldata";

	angular.module("noinfopath.drupal", [])
		.run(["$httpBackend", function($httpBackend) {
			if (angular.mock) $httpBackendRef = $httpBackend;
		}])

	.factory("carouselTemplate", [function() {
		var tmp = angular.element("<div></div>"),
			c = angular.element('<div id="carousel1" class="carousel slide" data-ride="carousel"></div>');

		tmp.append(c);

		c.append('<ol class="carousel-indicators"></ol>');
		c.append('<div class="carousel-inner" role="listbox"></div>');
		c.append('<a class="left carousel-control" href="#carousel1" role="button" data-slide="prev"> <span class="glyphicon glyphicon-chevron-left" aria-hidden="true"></span> <span class="sr-only">Previous</span> </a>');
		c.append('<a class="right carousel-control" href="#carousel1" role="button" data-slide="next"> <span class="glyphicon glyphicon-chevron-right" aria-hidden="true"></span> <span class="sr-only">Next</span> </a>');

		return tmp.html();
	}])

	.provider("drupalDataService", [function() {
		console.info("Using drupalDataService");


		function DataService($http, $timeout, endPointUrl) {
			var _endPointUrl = endPointUrl;


			Object.defineProperties(this, {
				defaultEndPointUrl: {
					get: function() {
						return _endPointUrl;
					},
					set: function(value) {
						_endPointUrl = value;
					}
				}
			});

			this.GET = function() {
				var req = {
						method: "GET",
						url: _endPointUrl,
						headers: {
							"Content-Type": "application/json",
							"Accept": "application/json"
						},
						withCredentials: true
					},
					promise = $http(req)
					.then(function(resp) {
						return resp.data;
					})
					.catch(function(err) {
						throw err;
					});

				if (angular.mock) $timeout($httpBackendRef.flush);

				return promise;
			};

		}

		this.$get = ["$http", "$timeout", function($http, $timeout) {
			return new DataService($http, $timeout, _defaultEndPointUrl);
		}];

	}])

	.provider("noDataService", [function() {

			this.setDefaultEndPointUrl = function(url){ _defaultEndPointUrl = url; };

		// function notImplemented(){
		// 	console.warn(defaultDataServiceName + " is not implemented.");
		// }
		//
		// function  NoDataService(){
		// 	var dataService;
		//
		// 	/*
		// 	*	#### @method configure
		// 	*/
		// 	this.configure = function(moduleName, serviceName){
		// 		var mn = moduleName ? moduleName : defaultModuleName,
		// 			sn = serviceName ? serviceName : defaultDataServiceName;
		//
		//
		// 		dataService = angular.injector(["ng", mn]).get(sn);
		//
		// 		this.GET = dataService.GET;
		// 	};
		//
		// 	this.GET =  notImplemented;
		// }

		this.$get = ["drupalDataService", function(drupalDataService) {
			return drupalDataService;
		}];

	}])

	/*
	 *	`http://recfinv2.img.local/carouseldata`
	 */
	.directive("noBootstrapCarousel", ["carouselTemplate", "noDataService", function(carouselTemplate, noDataService) {
		/*
		 *	#### @attribute module
		 *
		 *	This attribute, when provided, is the Anguar Module to load.
		 *	The default value is "noinfopath.drupal."
		 *
		 *	> NOTE: The approprieate JavaScript must already be loaded.
		 *
		 *	#### @attribute service
		 *
		 *	This attribute, when provided, is the name of the service to
		 *	inject.  It is required that the module already be loaded. This
		 *	detaults to using the `drupalDataService` service provider.
		 *
		 *	> See noDataService for more information.
		 *
		 *	#### @attribute service
		 *
		 *	This attribute, when provided, is the prefix for the data service
		 *	to bind to.  For example when `service` equals `drupal`, the
		 *	noDataService will inject a service with the name `drupalDataService`.
		 *
		 *	> See noDataService for more information.
		 *
		 */
		return {
			scope: true,
			link: function(scope, el, attrs) {
				el.html(carouselTemplate);

				function dataReady(data) {

					var indicators = el.find("OL"),
						slides = el.find(".carousel-inner"),
						tplLi = '<li data-target="#carousel1" data-slide-to="0"></li>',
						tplDiv = '<div class="item"> <img src="" alt=""> <div class="carousel-caption"></div> </div>';

					angular.forEach(data, function(o, k) {
						var li = angular.element(tplLi),
							item = angular.element(tplDiv),
							img = item.find("IMG"),
							caption = item.find(".carousel-caption");

						li.append(item);
						li.attr("data-slide-to", k);

						if (k === 0) {
							li.addClass("active");
							item.addClass("active");
						}


						img.attr("alt", o.title[0].value);
						img.attr("src", o.field_images[0].url);

						if (attrs.$attr.showCaption) caption.append(o.title[0].value);

						indicators.append(li);
						slides.append(item);

					});



					//console.log("dataReady", el.html());
				}

				function dataError(err) {
					throw err;
				}

				//console.log(carouselTemplate);
				// noDataService.configure();

				noDataService.GET()
					.then(dataReady)
					.catch(dataError);

			}
		};

	}]);


})(angular);
